<?php

/*
1.  Use this script to convert a web page using APL brace notation
    WSIS (Workspace Interchange Standard) to APL385 Unicode characters.

    If this script is called from the command line (locally), the incoming
    web page is assumed to be in Standard Input.  Use it as follows:

    php b2a.php <inputfile >outputfile

    If this script is called remotely (presumably, as an Apache Action
    handler), the incoming web page is assumed to be the file named in
    $_SERVER['PATH_TRANSLATED'].

    In both cases, the converted web page is written to Standard Output.

2.  The web page is expected to already mark the APL characters
    as belonging to the font family of either SImPL or "APL385 Unicode"
    in one way or another.  For example, you might use a Style Sheet
    which contains a class (say, "apl") as follows:

    <style type="text/css">
      .apl {font-family: "APL385 Unicode", monospace;}
    </style>

    and then surround inline APL characters with

    <span class="apl"> ... </span>

    or a block of APL characters with

    <div class="apl"> ... </div>

    etc.

    These fonts may be downloaded from

    http://www.vector.org.uk/resource/apl385.zip
    http://www.vector.org.uk/resource/simpl02.zip

3.  Use WSIS notation as in {rho} to represent the corresponding
    APL character.  This script will convert those five characters to
    &#9076; which is rho in APL385 Unicode.

    To display a left brace on a web page, use &#123; -- there is
    no need to encode specially a right brace as the algorithm below
    initially searches for a left brace only.

    Ensure that there are no spaces within the braces surrounding
    a keyword you want translated to an APL character.  For example

    { rho} and {rho } and { rho } will NOT be translated, but
    {rho} will.

4.  As alluded to above, this script may also be used as an Apache
    handler to convert on-the-fly web pages.  For example, you might
    use the extension .ahtml to designate a web page to be converted,
    and then tell Apache to convert such pages via this script by
    including in Apache's httpd.conf file the following lines:

    AddType    text/html .ahtml
    AddHandler Braces2APL .ahtml
    Action     Braces2APL /cgi-bin/b2a.php

    where /cgi-bin/ is the path to wherever you saved this file.

5.  The web page content is searched for a special statement of the form
    <meta name="Braces2APL" content="..." /> where the content portion
    contains a comma-separated list of keywords which describe special
    handling of the conversions.

    Keyword     Meaning
    ----------------------
    title       Surround the symbol with <span title="..."> </span>
                  where "..." is the symbol name.
    font=name   Surround the symbol with <span style="font-family: name">...</span>
    ***FIXME** -- isn't sensitve to "=name"

6.  This script was written by Bob Smith <bsmith@sudleyplace.com>,
    and is released under the GPL (GNU Public License).
*/

// Start of program
$PathToInc = 'www/cgi-bin';     // Do not start or end with a backslash

// Determine if we're being run locally (true) or remotely (false).
$Lcl = strlen ($_SERVER['REMOTE_ADDR']) == 0;

// If local, get the 'HOME' directory as we'll need
//   to prepend it to the .inc.php path
$Home = $Lcl ? $_ENV['HOME'] : '';

// Get the array of mappings of names to numbers
require ("$Home/$PathToInc/b2a.inc.php");

ob_implicit_flush (true); // Use unbuffered output

// Initialize various flags
$bTitle = false;    // TRUE iff surround Unicode symbol with <span title="..."> </span>
$bFont  = false;    // TRUE iff surround Unicode symbol with <span style="..."> </span>
$FontName = "APL385 Unicode";
$gFN = '';          // Global FileName

if ($Lcl)       // If we're called locally (hence standard input), ...
{
    $File = file_get_contents ('php://stdin');
    ProcessKeywords ($File);
    ProcessFile ($File, '');
} else          // Called remotely (via an Apache Action handler)
{
    $FileName = $_SERVER['PATH_TRANSLATED'];

    if (file_exists ($FileName))
    {
        $File = file_get_contents ($FileName);
        ProcessKeywords ($File);
        ProcessFile ($File, $FileName);
    } else
    {
        print // "HTTP/1.1 404 Not Found\n\n" .
              "<!doctype html PUBLIC \"-//IETF//DTD HTML 2.0//EN\">\n" .
              "<html>\n" .
              "<head>\n" .
              "<title>404 Not Found</title>\n" .
              "</head>\n" .
              "<body>\n" .
              "<h1>Not Found</h1>\n" .
              "The requested URL " .
              $_SERVER['REQUEST_URI'] .
              " was not found on this server.\n" .
              "</body>\n" .
              "</html>\n";
        fdne ($FileName);       // Display error message
    } // End IF/ELSE
} // End IF/ELSE

exit;


//*************************** FUNCTIONS ***********************//

// Process keywords in the incoming file
function ProcessKeywords ($File)
{
    global $bTitle, $bFont, $FontName;

    // Search for <meta name="Braces2APL" content="..." /> statement
    preg_match ('/<meta\s+name="Braces2APL"\s+content="(.*?)"\s*\/?\s*>/',
                $File,
                $Keywords);
    // Handle edge conditions
    if (count ($Keywords) == 0)
        return;
    $Keywords = $Keywords[1];       // Get the keywords
    if (strlen ($Keywords) == 0)
        return;
    $Keywords = explode (',', $Keywords);

    // Trim leading and trailing spaces
    $Keywords = preg_replace ('/^\s*(.*?)\s*$/', '$1', $Keywords);

    // Test for various keywords
    $bTitle = in_array ('title', $Keywords);
    $bFont  = in_array ('font', $Keywords);
} // End ProcessKeywords ()


// Process an entire file
function ProcessFile ($File, $FileName)
{
    global $br2a, $MaxKLen, $bTitle, $bFont, $FontName;

    // Initialize last Left Brace position
    $LastLBpos = 0;

    // Initialize output string
    $Out = '';

    // Search through the file for left braces
    while (false !== ($LBpos = strpos ($File, "{", $LastLBpos)))
    {
        // Output whatever preceded this left brace
        $Out .= substr ($File, $LastLBpos, $LBpos - $LastLBpos);

        // Find the matching right brace within a reasonable
        //   distance (length of longest keyword) and return the
        //   Character Entity number.  The "+2" after $MaxKLen
        //   accounts for the leading and trailing braces.
        if (false !== ($RBpos = strpos (substr ($File, $LBpos, $MaxKLen + 2), "}"))
         && !empty ($br2a[substr ($File, $LBpos + 1, $RBpos - 1)]))
        {
            $Name = substr ($File, $LBpos + 1, $RBpos - 1);
            $CENum = $br2a[$Name];

            // If both title and font keywords are specified,
            //   use one <span>
            if ($bTitle && $bFont)
                $Out .= "<span title=\"$Name\" style=\"font-family: '$FontName';\">";
            else
            // If we're to surround with title only, do so now
            if ($bTitle)
                $Out .= "<span title=\"$Name\">";
            else
            // If we're to surround with font only, do so now
            if ($bFont)
                $Out .= "<span style=\"font-family: '$FontName';\">";

            // Output the Character Entity
            $Out .= '&#' . $CENum . ';';

            if ($bTitle || $bFont)
                $Out .= "</span>";

            // Skip over the keyword and right brace
            $LastLBpos = $LBpos + $RBpos + 1;
        } else
        {
            // Print the left brace and skip over it
            $Out .= '{';
            $LastLBpos = $LBpos + 1;
        } // End IF/ELSE
    } // End WHILE

    // No more (or perhaps not any) left braces -- output the rest of the file
    $Out .= substr ($File, $LastLBpos);

    // Because Apache 1.x doesn't chain handlers, I substitute
    //   for the Server-Side Include statement as I use it as a
    //   common footer.
    echo ProcessIncludes ($Out, $FileName);
} // End ProcessFile ()


// Process includes
function ProcessIncludes ($Out, $FileName)
{
    global $gFN;

    $gFN = $FileName;   // Set global so EchoVar () can reference it
    $Out = preg_replace_callback ('/<!--#echo var=(\'|\")(.*?)\1\s*-->/',
                                  'EchoVar',
                                  $Out);
    return preg_replace_callback ('/<!--#include (virtual|file)="(.*?)"\s*-->/',
                                  'LoadFile',
                                  $Out);
} // End ProcessIncludes ()


// Simulate SSI <!--#echo var='...' -->
function EchoVar ($Matches)
{
    global $gFN;

    // $Matches[0] = Entire match
    //         [1] = Match for subpattern #1
    //         [2] = ...                  #2
    //         [3] = ...                  #3

    switch ($Matches[2])    // Split case on name of var
    {
        case 'LAST_MODIFIED':   // ***FIXME*** Make sensitive to <!--#config timefmt="..." -->
            return date ('d F Y H:i:s', filemtime ($gFN));

        default:
            return '*** Unknown #echo var (' . $Matches[2] . ') ***';
    } // End SWITCH
} // End EchoVar ()


// Load a file
function LoadFile ($Matches)
{
    // $Matches[0] = Entire match
    //         [1] = Match for subpattern #1
    //         [2] = ...                  #2
    //         [3] = ...                  #3

    $IncType = $Matches[1];     // Save include type: 'virtual' or 'file'
    $Matches = $Matches[2];     // We need this entry only

    // Check for '/gcounter.cgi?'
    $gDomain = preg_replace ('/\/gcounter.cgi\?domain=(.*)/', '$1', $Matches);
    if ($gDomain !== $Matches)
    {
        include_once ('gcounter.php');
        return '';
    } else
    {
        // Prepend a slash if not already present
        if ($Matches{0} != '/')
            $Matches = '/' . $Matches;

        // Construct the path/filename
        $FileName = $_SERVER['DOCUMENT_ROOT']
                  . (($IncType == 'file')
                    ? preg_replace ('!(.*/).*?!', '$1', $_SERVER['DOCUMENT_URI'])
                    : '')
                  . $Matches;

        // Eliminate multiple slashes
        $FileName = preg_replace ('!//+!', '/', $FileName);

        if ($Matches == '/ValidXHTML.html'
         || $Matches == '/ValidXHTML-CSS.html')
            $FileName = '/www/htdocs' . $Matches;

        if (file_exists ($FileName))
            return ProcessIncludes (file_get_contents ($FileName), $FileName);
        else
        {
            fdne ($FileName);       // Display error message
            return '';
        } // End IF/ELSE
    } // End IF/ELSE
} // End LoadFile ()


// Display "File does not exist" error message
function fdne ($FileName)
{
    $stderr = fopen ('php://stderr', 'w');
    fwrite ($stderr, "[" .
                      date ('D M d H:i:s Y') .
                     "] [error] [client " .
                     $_SERVER['REMOTE_ADDR'] .
                     "] File does not exist: $FileName\n");
    fclose ($stderr);
} // End fdne ()


?>
