#!/usr/bin/perl

#  1.  Use this script to convert a web page using APL brace notation
#	   WSIS (Workspace Interchange Standard) to APL385 Unicode characters.
#
#	   If this script is called from the command line (locally), the incoming
#	   web page is assumed to be in Standard Input.  Use it as follows:
#
#	   perl b2a.pl <inputfile >outputfile
#
#	   If this script is called remotely (presumably, as an Apache Action
#	   handler), the incoming web page is assumed to be the file named in
#	   $ENV{'PATH_TRANSLATED'}.
#
#	   In both cases, the converted web page is written to Standard Output.
#
#  2.  The web page is expected to already mark the APL characters
#	   as belonging to the font family of either SImPL or "APL385 Unicode"
#	   in one way or another.  For example, you might use a Style Sheet
#	   which contains a class (say, "apl") as follows:
#			
#	   <style type="text/css">
#		 .apl {font-family: "APL Unicode";}
#	   </style>
#
#	   and then surround inline APL characters with
#
#	   <span class="apl"> ... </span>
#
#	   or a division of APL characters with
#			
#	   <div class="apl"> ... </div>
#
#	   etc.
#
#	   These fonts may be downloaded from
#
#	   http://www.vector.org.uk/resource/apl385.zip
#	   http://www.vector.org.uk/resource/simpl02.zip
#
#  3.  Use WSIS notation as in {rho} to represent the corresponding
#	   APL character.  This script will convert those five characters to
#	   &#9076; which is rho in APL385 Unicode.
#
#	   To display a left brace on a web page, use &#123; -- there is
#	   no need to encode specially a right brace as the algorithm below
#	   initially searches for a left brace only.
#
#	   Ensure that there are no spaces within the braces surrounding
#	   a keyword you want translated to an APL character.  For example
#
#	   { rho} and {rho } and { rho } will not be translated, but
#	   {rho} will.
#
#  4.  As alluded to above, this script may also be used as an Apache
#	   handler to convert on-the-fly web pages.  For example, you might
#	   use the extension .ahtml to designate a web page to be converted,
#	   and then tell Apache to convert such pages via this script by
#	   including in Apache's httpd.conf file the following lines:
#
#	   AddHandler brace-to-unicode .ahtml
#	   Action	  brace-to-unicode /cgi-bin/b2a.php
#	   AddType	  text/html .ahtml
#
#	   where /cgi-bin/ is the path to wherever you saved this file.
#
#  5.  This script was written by Bob Smith <bsmith@sudleyplace.com>,
#	   and is released under the GPL (GNU Public License).

use 	strict;

# Start of program
$| = 1; 		# Use unbuffered output
print "Content-type: text/html\n\n";

# Get the array of mappings of names to numbers
require "b2a.ph";
use vars qw (%br2a $MaxKLen);

# Determine if we're being run locally (true) or remotely (false).
my $Lcl = (defined $ENV{'REMOTE_ADDR'}) ? 0 : 1;

if ($Lcl)		# If we're called locally (hence standard input), ...
{
	my $Line;
	while ($Line = <STDIN>)
	{
		&ProcessLine ($Line);
	} # End WHILE
} else			# Called remotely (via an Apache Action handler)
{
	my $FileName = $ENV{'PATH_TRANSLATED'};

	if (-e $FileName)
	{
		my $FileSize = -s $FileName;
		my $Line;
		open INPFILE, $FileName;
		read INPFILE, $Line, $FileSize;
		close INPFILE;

		&ProcessLine ($Line);
	} else
	{
		print # "HTTP/1.1 404 Not Found\n\n" .
			  "<!doctype html PUBLIC \"-//IETF//DTD HTML 2.0//EN\">\n" .
			  "<html>\n" .
			  "<head>\n" .
			  "<title>404 Not Found</title>\n" .
			  "</head>\n" .
			  "<body>\n" .
			  "<h1>Not Found</h1>\n" .
			  "The requested URL " .
			  $ENV{'REQUEST_URI'} .
			  " was not found on this server.\n" .
			  "</body>\n" .
			  "</html>\n";
		&fdne ($FileName);		# Display error message
	} # End IF/ELSE
} # End IF/ELSE

exit;


#*************************** FUNCTIONS ***********************

# Process a line (possibly an entire file)
sub ProcessLine ($Line)
{
	my ($Line) = @_;
	my ($LBpos, $RBpos);

	# Initialize last Left Brace position
	my $LastLBpos = 0;

	# Initialize output string
	my $Out = '';

	# Search through the line for left braces
	while (-1 != ($LBpos = index $Line, "{", $LastLBpos))
	{
		# Output whatever preceded this left brace
		$Out .= substr $Line, $LastLBpos, $LBpos - $LastLBpos;

		# Find the matching right brace within a reasonable
		#	distance (length of longest keyword) and return the
		#	Character Entity number.  The "+2" after $MaxKLen
		#	accounts for the leading and trailing braces.
		if (-1 != ($RBpos = index (substr ($Line, $LBpos, $MaxKLen + 2), "}"))
		 && defined $br2a{substr $Line, $LBpos + 1, $RBpos - 1})
		{
			# Output the Character Entity
			$Out .= '&#' . $br2a{substr $Line, $LBpos + 1, $RBpos - 1} . ';';

			# Skip over the keyword and right brace
			$LastLBpos = $LBpos + $RBpos + 1;
		} else
		{
			# Print the left brace and skip over it
			$Out .= '{';
			$LastLBpos = $LBpos + 1;
		} # End IF/ELSE
	} # End WHILE

	# No more (or perhaps not any) left braces -- output the rest of the line
	$Out .= substr $Line, $LastLBpos;

	# Because Apache 1.x doesn't chain handlers, I substitute
	#	for the Server-Side Include statement as I use it as a
	#	common footer.
	my $pat = "(.*?)<!--\\s*#include virtual=\"(.*?)\"\\s*-->(.*)";
	while ($Out =~ /$pat/gmsi)
	{
		print $1;
		my $FileName = $ENV{'DOCUMENT_ROOT'} . '/' . $2;
		$Out = $3;

		if (-e $FileName)
		{
			my $FileSize = -s $FileName;
			my $File;
			open INPFILE, $FileName;
			read INPFILE, $File, $FileSize;
			close INPFILE;

			print $File;
		} else
		{
			&fdne ($FileName);	# Display error message
		} # End IF/ELSE
	} # End WHILE

	# If we matched $pat in $Out, print the rest of the string,
	#	otherwise, print the entire string.
	print $Out;
} # End ProcessLine ()


# Display "File does not exist" error message
sub fdne
{
	my ($FileName) = @_;

	print (STDERR "[" .
				   (localtime) .
				  "] [error] [client " .
				  $ENV{'REMOTE_ADDR'} .
				  "] File does not exist: $FileName\n");
} # End fnde ()

